/*
 * LoggerHandler.java
 *
 * Created on 16 May 2009
 *
 * Version 1.2
 *
 * Copyright (c) Kieran Greer
 */

package org.jlog2;


import java.util.Vector;


/**
 * This class handles logging operations.
 * @author Kieran Greer
 */
public class LoggerHandler
{
    /** Logging levels */
    /** Indicates all logging levels */
    public static final String ALL = "All";

    /** The info log level */
    public static final String INFO = "Info";

    /** The warn log level */
    public static final String WARN = "Warn";

    /** The error log level */
    public static final String ERROR = "Error";

    /** The severe log level */
    public static final String SEVERE = "Severe";

    /** The message log level - always output to stdout and any others */
    public static final String MESSAGE = "Message";


    /** The order for the logging levels */
    private static Vector logOrder;

    /** True if user logger set, false otherwise */
    private static boolean userLogger;

    /** An object to synchronize on */
    public static final Object syncOn;

    /** The factory that generates the loggers */
    protected static LoggerFactory loggerFactory;


    static
    {
        syncOn = new Object();
        logOrder = new Vector();
        userLogger = false;

        logOrder = new Vector();
        logOrder.addElement(ALL.toLowerCase());
        logOrder.addElement(INFO.toLowerCase());
        logOrder.addElement(WARN.toLowerCase());
        logOrder.addElement(ERROR.toLowerCase());
        logOrder.addElement(SEVERE.toLowerCase());
        logOrder.addElement(MESSAGE.toLowerCase());

        try
        {
            loggerFactory = new CustomLoggerFactory();
        }
        catch (Exception ex)
        {}
    }

    
    /** 
     * Creates a new instance of LoggerHandler.
     * @throws java.lang.Exception any error.
     */
    public LoggerHandler() throws Exception
    {}

    /**
     * Creates a new instance of LoggerHandler.
     * @param theLoggerFactory this can be a new logger factory you have written
     * to override the operation of the default CustomLoggerFactory.
     * @throws java.lang.Exception any error.
     */
    public LoggerHandler(LoggerFactory theLoggerFactory) throws Exception
    {
        loggerFactory = theLoggerFactory;
    }

    /**
     * Set the user logger factory to control the logging channels.
     * @param userLoggerFactory the factory to generate new loggers with.
     */
    public static void setUserLoggerFactory(LoggerFactory userLoggerFactory)
    {
        if (!userLogger)
        {
            loggerFactory = userLoggerFactory;
        }
    }

    /**
     * Return a logger appropriate to the class specified.
     * @param loggerID the id used to identify the logger.
     * @return the global logger by default in this implementation.
     */
    public static Logger getLogger(String loggerID)
    {
        return loggerFactory.getLoggerInstance(loggerID);
    }

    /**
     * Add default channels to write the log messages to.
     * @param theLogChannels the channels to write the log messages to.
     */
    public static void addDefaultLogChannels(Vector theLogChannels)
    {
        loggerFactory.addDefaultLogChannels(theLogChannels);
    }

    /**
     * Add default channels to write the debug messages to.
     * @param theLogChannels the channels to write the debug messages to.
     */
    public static void addDefaultDebugChannels(Vector theLogChannels)
    {
        loggerFactory.addDefaultDebugChannels(theLogChannels);
    }

    /**
     * Return true if the level to log at is higher or equal to the configuration level.
     * @param theLevel the level the error or log message is at.
     * @param logLevel the level the configuration is set to log to.
     * @return true if theLevel is higher or equal to logLevel.
     */
    public static boolean levelOK(String theLevel, String logLevel)
    {
        if ((theLevel != null) && (logLevel != null))
        {
            theLevel = theLevel.toLowerCase();
            logLevel = logLevel.toLowerCase();

            if (logOrder.contains(theLevel) && logOrder.contains(logLevel))
            {
                return (logOrder.indexOf(theLevel) >= logOrder.indexOf(logLevel));
            }
        }

        return false;
    }

    /**
     * Log the message on the logger if suitable.
     * @param logger the logger to write to.
     * @param level the error level.
     * @param methodName the name of the method in the class throwing the exception.
     * @param message the message to log.
     */
    public static void logMessage(Logger logger, String level, String methodName,
            String message)
    {
        synchronized(syncOn)
        {
            try
            {
                logger.logMessage(level, methodName, message);
            }
            catch (Exception ex) {}
        }
    }

    /**
     * Log the message on the debug stream if suitable.
     * @param logger the logger to write to.
     * @param level the error level.
     * @param methodName the name of the method in the class throwing the exception.
     * @param message the message to log.
     */
    public static void debugMessage(Logger logger, String level, String methodName,
            String message)
    {
        synchronized(syncOn)
        {
            try
            {
                logger.debugMessage(level, methodName, message);
            }
            catch (Exception ex) {}
        }
    }

    /**
     * Log the error on the log stream.
     * @param logger the logger to write to.
     * @param level the error level.
     * @param error the error to log.
     */
    public static void logError(Logger logger, String level, Exception error)
    {
        synchronized(syncOn)
        {
            try
            {
                logger.logError(level, error);
            }
            catch (Exception ex) {}
        }
    }

    /**
     * Log the error on the debug log stream.
     * @param logger the logger to write to.
     * @param level the error level.
     * @param error the error to log.
     */
    public static void debugError(Logger logger, String level, Exception error)
    {
        synchronized(syncOn)
        {
            try
            {
                logger.debugError(level, error);
            }
            catch (Exception ex) {}
        }
    }
}
